/*****************************************************************************
	AmyEdit - A GTKMM LaTeX Editor
	Copyright (C) 2004-2005 Rob Page
	email:  page <dot> rob <at> gmail <dot> com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "PrefStruct.hh"
#include "Utils.hh"

#include <glibmm/miscutils.h>

PrefStruct* PrefStruct::smInstance=NULL;

PrefStruct* PrefStruct::instance()
{
	static bool running = false;
	if (smInstance == NULL && running == false)
	{
		running = true;
		smInstance = new PrefStruct();
		running = false;
	}
	return smInstance;
}
                                                 
void PrefStruct::goodnight()
{
	if (smInstance!=NULL)
	{
		delete smInstance;
	}
}


PrefStruct::PrefStruct()
{

	std::string config_filename = Glib::build_filename("amyedit", "amyedit.conf");
	std::string full_path = "";

	std::list<std::string> list = Glib::ArrayHandle<std::string>(g_get_system_config_dirs());
	list.push_back(g_get_user_config_dir());

	std::list<std::string> config_files;
	std::list<std::string> default_files;
	
	std::list<std::string>::iterator iter;
	for( iter = list.begin(); iter != list.end(); iter++ )
	{
		full_path = Glib::build_filename(*iter, config_filename);
		if (Glib::file_test(full_path, Glib::FILE_TEST_EXISTS))
		{
			config_files.push_back(full_path);
		}
	}

	config_filename = Glib::build_filename("amyedit", "default_config");
	list = Glib::ArrayHandle<std::string>(g_get_system_data_dirs());
	list.push_back(g_get_user_data_dir());

	for( iter = list.begin(); iter != list.end(); iter++ )
	{
		full_path = Glib::build_filename(*iter, config_filename);
		if (Glib::file_test(full_path, Glib::FILE_TEST_EXISTS))
		{
			config_files.push_back(full_path);
			default_files.push_back(full_path);
		}
	}
	
	if (config_files.empty())
	{
		Amy::Message(_("Failed to find a usable config file"),
	    	         _("This probably means AmyEdit is not properly installed."),
	        	     Amy::ERROR);
		throw Glib::FileError(Glib::FileError::NO_SUCH_ENTITY, "AmyEdit Configuration File Not Found");
	}

	for(iter = config_files.begin(); iter != config_files.end(); iter++)
	{
		/* TODO This is so evil it really shouldn't be here */
		try {
			if (config.load_from_file(*iter))
				break;
		} catch (...) { /* Do nothing */ }
			Amy::Message(_("Failed to load config File ") + full_path,
			             _("This probably means AmyEdit is not properly installed..."),
			             Amy::ERROR);
	}
	if (iter == config_files.end())
	{
		Amy::Message(_("Could not find a valid config file"),
		             _("This probably means AmyEdit is not properly installed."),
					 Amy::ERROR);
		/* TODO Have an AmyEdit Exception or something */
		throw Glib::FileError(Glib::FileError::NO_SUCH_ENTITY, "No Valid Config File Found");
	}
	
	if (default_files.empty())
	{
		Amy::Message(_("Failed to find a usable default config file"),
	    	         _("This probably means AmyEdit is not properly installed."),
	        	     Amy::ERROR);
		throw Glib::FileError(Glib::FileError::NO_SUCH_ENTITY, "AmyEdit Configuration File Not Found");
	}

	for(iter = default_files.begin(); iter != default_files.end(); iter++)
	{
		/* TODO This is so evil it really shouldn't be here */
		try {
			if (defaults.load_from_file(*iter))
				break;
		} catch (...) { /* Do nothing */ }
			Amy::Message(_("Failed to load a default config file ") + full_path,
			             _("This probably means AmyEdit is not properly installed..."),
			             Amy::ERROR);
	}
	if (iter == default_files.end())
	{
		Amy::Message(_("Could not find a valid default config file"),
		             _("This probably means AmyEdit is not properly installed."),
					 Amy::ERROR);
		/* TODO Have an AmyEdit Exception or something */
		throw Glib::FileError(Glib::FileError::NO_SUCH_ENTITY, "No Valid Config File Found");
	}
	
	/* TODO Make this be nicer */
	if (!has_key("General", "config_dir"))
		set("General", "config_dir", Glib::filename_to_utf8(Glib::build_filename(g_get_user_config_dir(), "amyedit")));
	
	if (!has_key("General", "data_dir"))
		set("General", "data_dir", Glib::filename_to_utf8(Glib::build_filename(g_get_user_data_dir(), "amyedit")));
}

PrefStruct::~PrefStruct()
{
	Glib::ustring config_filename = Glib::build_filename(g_get_user_config_dir(), Glib::build_filename("amyedit", "amyedit.conf"));
	try
	{
		Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(config_filename, "w");
		outfile->write(config.to_data());
		outfile->flush();
	}
	catch(Glib::Error error)
	{
		Amy::Message(_("Failed to save configuration file"), _("An error occurred whilst attempting to save the file ") + config_filename + _(", this file stores your preferences. The system reported this error as: ") + error.what(), Amy::ERROR);
	}
}

Glib::ustring PrefStruct::get(const Glib::KeyFile& keyfile, const Glib::ustring& group_name, const Glib::ustring& key, Glib::ustring* nullpointer) const
{
	return keyfile.get_string(group_name, key);
}

int PrefStruct::get(const Glib::KeyFile& keyfile, const Glib::ustring& group_name, const Glib::ustring& key, int* nullpointer) const
{
	return keyfile.get_integer(group_name, key);
}

bool PrefStruct::get(const Glib::KeyFile& keyfile, const Glib::ustring& group_name, const Glib::ustring& key, bool* nullpointer) const
{
	return keyfile.get_boolean(group_name, key);
}

Glib::ArrayHandle<Glib::ustring> PrefStruct::get(const Glib::KeyFile& keyfile, const Glib::ustring& group_name, const Glib::ustring& key, Glib::ArrayHandle<Glib::ustring>* nullpointer) const
{
	return keyfile.get_string_list(group_name, key);
}

void PrefStruct::set(const Glib::ustring& group_name, const Glib::ustring& key, const Glib::ustring& value)
{
	config.set_string(group_name, key, value);
}

void PrefStruct::set(const Glib::ustring& group_name, const Glib::ustring& key, int value)
{
	config.set_integer(group_name, key, value);
}

void PrefStruct::set(const Glib::ustring& group_name, const Glib::ustring& key, bool value)
{
	config.set_boolean(group_name , key, value);
}

void PrefStruct::set(const Glib::ustring& group_name , const Glib::ustring& key, Glib::ArrayHandle<Glib::ustring> value)
{
	config.set_string_list(group_name, key, value);
}

Glib::ArrayHandle<Glib::ustring> PrefStruct::get_keys(const Glib::ustring& group_name) const
{
	return config.get_keys(group_name);
}

void PrefStruct::remove_key(const Glib::ustring& group_name, const Glib::ustring& key)
{
	config.remove_key(group_name, key);
}

bool PrefStruct::has_key(const Glib::ustring& group_name, const Glib::ustring& key) const
{
	return config.has_key(group_name, key);
}
