/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "TextListViewListBox.hh"

TextListViewListBox::TextListViewListBox(int columns)
{
	set_model(TextListModel::create(columns));

	get_selection()->set_mode(Gtk::SELECTION_SINGLE);
}

void TextListViewListBox::show_column(const Glib::ustring& title, int column)
{
	append_column(title, get_model()->get_column(column));
}

void TextListViewListBox::select_first(){
  Gtk::TreeModel::iterator iter = get_model()->children().begin();
  if(iter)
    get_selection()->select(iter);
}

void TextListViewListBox::select(const Glib::ustring& selection)
{
	get_selection()->select(get_model()->find_value(selection));
}

Glib::ustring TextListViewListBox::get_selected(int column)
{
	Gtk::TreeModel::iterator iter = get_selection()->get_selected();
	if (iter)
	{
		return get_model()->get_value(iter, column);
	} 
	else
	{
		return Glib::ustring("");
	}
}

Glib::RefPtr<TextListModel> TextListViewListBox::get_model()
{
	return Glib::RefPtr<TextListModel>::cast_dynamic(Gtk::TreeView::get_model());
}
