#ifndef _GTKSOURCEVIEWMM_SOURCEBUFFER_H
#define _GTKSOURCEVIEWMM_SOURCEBUFFER_H

#include <glibmm.h>
#include <gdkmm/pixbuf.h>
/* sourcebuffer.h
 * 
 * Based on gtksourceviewmm by Jae Jang
 * Copyright (C) 2005 Rob Page
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtkmm/textbuffer.h>
#include <gtkmm/textview.h>
#include <gtksourceview/gtksourcetagstyle.h>
#include <gtksourceviewmm/sourcelanguage.h>
#include <gtksourceviewmm/sourcelanguagesmanager.h>

#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef struct _GtkSourceBuffer GtkSourceBuffer;
typedef struct _GtkSourceBufferClass GtkSourceBufferClass;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */


namespace Gtk
    { class SourceBuffer_Class; } // namespace Gtk
namespace Gtk
{

class SourceBuffer : public Gtk::TextBuffer
{
public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    typedef SourceBuffer CppObjectType;
    typedef SourceBuffer_Class CppClassType;
    typedef GtkSourceBuffer BaseObjectType;
    typedef GtkSourceBufferClass BaseClassType;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

    virtual ~SourceBuffer();

#ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
    friend class SourceBuffer_Class;
    static CppClassType sourcebuffer_class_;

    // noncopyable
    SourceBuffer(const SourceBuffer&);
    SourceBuffer& operator=(const SourceBuffer&);

protected:
    explicit SourceBuffer(const Glib::ConstructParams& construct_params);
    explicit SourceBuffer(GtkSourceBuffer* castitem);

#endif /* DOXYGEN_SHOULD_SKIP_THIS */

public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    static GType get_type()      G_GNUC_CONST;
    static GType get_base_type() G_GNUC_CONST;
#endif

    ///Provides access to the underlying C GtkObject.
    GtkSourceBuffer*       gobj()       { return reinterpret_cast<GtkSourceBuffer*>(gobject_); }

    ///Provides access to the underlying C GtkObject.
    const GtkSourceBuffer* gobj() const { return reinterpret_cast<GtkSourceBuffer*>(gobject_); }


public:
    //C++ methods used to invoke GTK+ virtual functions:

protected:
    //GTK+ Virtual Functions (override these to change behaviour):

private:

public:

    SourceBuffer();

    Glib::RefPtr <Gtk::SourceLanguagesManager> get_languages_manager();

    void set_check_brackets (bool check=true);
    bool get_check_brackets ();

    void set_bracket_match_style(GtkSourceTagStyle *style);

    void set_highlight (bool highlight=true);
    bool get_highlight ();

    void set_max_undo_levels(int max_undo_levels);
    int get_max_undo_levels();

    void set_language (Glib::RefPtr<Gtk::SourceLanguage> language);
	Glib::RefPtr<Gtk::SourceLanguage> get_language();

    void set_escape_char (gunichar escape_char);
    gunichar get_escape_char ();

    bool can_undo();
    bool can_redo();

    void undo();
    void redo();

    void begin_not_undoable_action();
    void end_not_undoable_action();

    /* FIXME Marker stuff missing... */

};

} // namespace Gtk


namespace Glib
{
Gtk::SourceBuffer* wrap(GtkSourceBuffer* object, bool take_copy = false);
}
#endif 
