/*****************************************************************************
	AmyEdit - A GTKMM LaTeX Editor
	Copyright (C) 2004-2005 Rob Page
	email:  page <dot> rob <at> gmail <dot> com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include <gtkmm/main.h>
#include <sys/stat.h>
#include <glib/gstdio.h>
#include <fstream>

#include "AmyEdit.hh"
#include "gtksourceviewmm/init.hh"
#include "gtkmm_extra/keyfile.h"
#include "Utils.hh"

int main(int argc, char **argv)
{  
	bindtextdomain (GETTEXT_PACKAGE, AMYEDITLOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
	
	Gtk::Main m(&argc, &argv);
	Gtk::SourceViewmm_init();
	Glib::keyfile_init();

	/* TODO This should probably be removed and created on demand */
	
	// make directory structure
	std::string config_dir = Glib::build_filename(g_get_user_config_dir(), "amyedit");
	g_mkdir(g_get_user_config_dir(), S_IRWXU);
	g_mkdir(config_dir.c_str(),S_IRWXU);

	std::string data_dir = g_get_user_data_dir();
	std::string amyedit_data_dir = Glib::build_filename(data_dir, "amyedit");
	
	g_mkdir(Glib::path_get_dirname(data_dir).c_str(), S_IRWXU);
	g_mkdir(data_dir.c_str(), S_IRWXU);
	g_mkdir(amyedit_data_dir.c_str(),S_IRWXU);
	g_mkdir(Glib::build_filename(amyedit_data_dir,"templates").c_str(),S_IRWXU);

	const Glib::ustring bug_message = "This is a bug! Please report it to page.rob@gmail.com, including the message below, and what you were doing at the time of the crash.\n\nThe system reported this as : ";
	
	try
	{
		int i;
		for (i=1; argv[i]; i++)
			AmyEdit::instance()->open(Glib::ustring(argv[i]));
	} 
	catch (Glib::Error e)
	{
		Amy::Message("Something went wrong whilst starting up AmyEdit", bug_message + e.what(), Amy::ERROR);
		throw e;
	}

	try
	{
		m.run(*AmyEdit::instance());
	}
	catch (Glib::Error e)
	{
		Amy::Message("Something went wrong whilst running AmyEdit", bug_message + e.what(), Amy::ERROR);
		throw e;
	}

	try
	{
		AmyEdit::goodnight();
	}
	catch (Glib::Error e)
	{
		Amy::Message("Something went wrong whilst closing down AmyEdit", bug_message + e.what(), Amy::ERROR);
		throw e;
	}

	return 0;
}

